#Link CPs - Surface Variables (Iberian Peninsula and Catalunya)
#Plotting spatial patterns (anom)

#AEMET 5km

library(ncdf4)
library(ggplot2)
library(gridExtra)
library(fields)
library(mapdata)
#library(matrixStats)
#library(metR)
library(RColorBrewer)

rm(list=ls()) ; gc()

#Directory information
setwd("/esarchive/scratch/molmo/")
OUTPUTS <- "/esarchive/scratch/molmo/SpatialPatterns_AEMET-5km/"

files <- list.files(pattern="AEMET-5km_CPs.Rdata")

for(f in 1:length(files)){
  message(paste(f," out of ",length(files),sep=""))
  load(files[f])
  k = length(unique(datitos$CP))
  dom=as.character(unique(datitos$Domain))
  var = substr(files[f],start=30,stop=31)

if(var=="PR"){
datitos$Z[which(datitos$Z > 5)] <- 4.999
datitos$Z[which(datitos$Z < -5)] <- -4.999

p <- ggplot(datitos,aes(x=Longitude,y=Latitude))  + geom_tile(aes(fill=Z))  + borders(colour = "grey22",size=0.3) + facet_wrap(~CP,ncol=k)+ 
  scale_fill_gradientn(name=expression("PR (mm)"),breaks=seq(-5,5,by=2),limits=c(-5,5),
                       colours=brewer.pal(9,"BrBG"),aesthetics=c("fill"),
                       na.value = "white") +
  coord_quickmap(xlim=c(min(datitos$Longitude),max(datitos$Longitude)),ylim=c(min(datitos$Latitude),max(datitos$Latitude)),expand = F) +
  xlab("Longitude") + ylab("Latitude") + theme_bw() 
#p <- p + geom_rect(aes(xmin=x1,xmax=x2,ymin=y1,ymax=y2),col="red",fill=NA)
p <- p + geom_rect(aes(xmin=0,xmax=4,ymin=40,ymax=43),col="black",fill=NA,alpha=0.003)
ggsave(p,filename=paste(OUTPUTS,"SpatialPatterns_Ward_Surface_PR_Anom_",dom,"_Iberia_",k,"CPs_AEMET-5km.png",sep=""),device="png",
       dpi=600,width=2*k,height = 3)

p <- ggplot(subset(datitos,Latitude>40 & Latitude < 43 & Longitude > 0 & Longitude < 4),aes(x=Longitude,y=Latitude))  + geom_tile(aes(fill=Z))  + borders(colour = "grey22",size=0.3) + facet_wrap(~CP,ncol=k)+ 
  scale_fill_gradientn(name=expression("PR (mm)"),breaks=seq(-5,5,by=2),limits=c(-5,5),
                       colours=brewer.pal(9,"BrBG"),aesthetics=c("fill"),
                       na.value = "white") +
  coord_quickmap(xlim=c(min(datitos$Longitude),max(datitos$Longitude)),ylim=c(min(datitos$Latitude),max(datitos$Latitude)),expand = F) +
  xlab("Longitude") + ylab("Latitude") + theme_bw() 
#p <- p + geom_rect(aes(xmin=x1,xmax=x2,ymin=y1,ymax=y2),col="red",fill=NA)
p <- p + geom_rect(aes(xmin=0,xmax=4,ymin=40,ymax=43),col="black",fill=NA,alpha=0.003)
ggsave(p,filename=paste(OUTPUTS,"SpatialPatterns_Ward_Surface_PR_Anom_",dom,"_Catalunya_",k,"CPs_AEMET-5km.png",sep=""),device="png",
       dpi=600,width=2*k,height = 3)
}

if(var=="TN"){
  datitos$Z[which(datitos$Z > 3)] <- 2.999
  datitos$Z[which(datitos$Z < -3)] <- -2.999
  
  p <- ggplot(datitos,aes(x=Longitude,y=Latitude))  + geom_tile(aes(fill=Z))  + borders(colour = "grey22",size=0.3) + facet_wrap(~CP,ncol=k)+ 
    scale_fill_gradientn(name=expression("TN (ºC)"),breaks=seq(-3,3,by=1),limits=c(-3,3),
                         colours=rev(brewer.pal(9,"RdBu")),aesthetics=c("fill"),
                         na.value = "white") +
    coord_quickmap(xlim=c(min(datitos$Longitude),max(datitos$Longitude)),ylim=c(min(datitos$Latitude),max(datitos$Latitude)),expand = F) +
    xlab("Longitude") + ylab("Latitude") + theme_bw()
  #p <- p + geom_rect(aes(xmin=x1,xmax=x2,ymin=y1,ymax=y2),col="red",fill=NA)
  p <- p + geom_rect(aes(xmin=0,xmax=4,ymin=40,ymax=43),col="black",fill=NA,alpha=0.003)
  ggsave(p,filename=paste(OUTPUTS,"SpatialPatterns_Ward_Surface_TN_Anom_",dom,"_Iberia_",k,"CPs_AEMET-5km.png",sep=""),device="png",
         dpi=600,width=2*k,height = 3)
  
  p <- ggplot(subset(datitos,Latitude>40 & Latitude < 43 & Longitude > 0 & Longitude < 4),aes(x=Longitude,y=Latitude))  + geom_tile(aes(fill=Z))  + borders(colour = "grey22",size=0.3) + facet_wrap(~CP,ncol=k)+ 
    scale_fill_gradientn(name=expression("TN (ºC)"),breaks=seq(-3,3,by=1),limits=c(-3,3),
                         colours=rev(brewer.pal(9,"RdBu")),aesthetics=c("fill"),
                         na.value = "white") +
    coord_quickmap(xlim=c(0,4),ylim=c(40,43),expand = F) +
    xlab("Longitude") + ylab("Latitude") + theme_bw() 
  ggsave(p,filename=paste(OUTPUTS,"SpatialPatterns_Ward_Surface_TN_Anom_",dom,"_Catalunya_",k,"CPs_AEMET-5km.png",sep=""),device="png",
         dpi=600,width=2*k,height = 3)
}  
  
  if(var=="TX"){
    datitos$Z[which(datitos$Z > 3)] <- 2.999
    datitos$Z[which(datitos$Z < -3)] <- -2.999
    
    p <- ggplot(datitos,aes(x=Longitude,y=Latitude))  + geom_tile(aes(fill=Z))  + borders(colour = "grey22",size=0.3) + facet_wrap(~CP,ncol=k)+ 
      scale_fill_gradientn(name=expression("TX (ºC)"),breaks=seq(-3,3,by=1),limits=c(-3,3),
                           colours=rev(brewer.pal(9,"RdBu")),aesthetics=c("fill"),
                           na.value = "white") +
      coord_quickmap(xlim=c(min(datitos$Longitude),max(datitos$Longitude)),ylim=c(min(datitos$Latitude),max(datitos$Latitude)),expand = F) +
      xlab("Longitude") + ylab("Latitude") + theme_bw()  
    #p <- p + geom_rect(aes(xmin=x1,xmax=x2,ymin=y1,ymax=y2),col="red",fill=NA)
    p <- p + geom_rect(aes(xmin=0,xmax=4,ymin=40,ymax=43),col="black",fill=NA,alpha=0.003)
    ggsave(p,filename=paste(OUTPUTS,"SpatialPatterns_Ward_Surface_TX_Anom_",dom,"_Iberia_",k,"CPs_AEMET-5km.png",sep=""),device="png",
           dpi=600,width=2*k,height = 3)
    
    p <- ggplot(subset(datitos,Latitude>40 & Latitude < 43 & Longitude > 0 & Longitude < 4),aes(x=Longitude,y=Latitude))  + geom_tile(aes(fill=Z))  + borders(colour = "grey22",size=0.3) + facet_wrap(~CP,ncol=k)+ 
      scale_fill_gradientn(name=expression("TX (ºC)"),breaks=seq(-3,3,by=1),limits=c(-3,3),
                           colours=rev(brewer.pal(9,"RdBu")),aesthetics=c("fill"),
                           na.value = "white") +
      coord_quickmap(xlim=c(0,4),ylim=c(40,43),expand = F) +
      xlab("Longitude") + ylab("Latitude") + theme_bw() 
    ggsave(p,filename=paste(OUTPUTS,"SpatialPatterns_Ward_Surface_TX_Anom_",dom,"_Catalunya_",k,"CPs_AEMET-5km.png",sep=""),device="png",
           dpi=600,width=2*k,height = 3)
  }    
}
