library(loadeR)
library(transformeR)
library(ggplot2)
library(RColorBrewer)
library(maps)
#library(climate4R.value)
library(Kendall)
library(trend)

rm(list=ls()) ; graphics.off() ; gc()

#Directory information
setwd("~/Downloads/ETCCDI_aemet-5km/")
OUTPUTS <- "../OUTPUTS/"

#Useful information
ETCCDI_index <- c("Rx1day","Rx5day","TN10p","TN90p","TNn","TNx","TX10p","TX90p","TXn","TXx","CDD","CWD","CSDI",
                  "PRCPTOT","TR","R10mm","R20mm","R95pTOT","WSDI")
seasons <- list(Summer=c(6,7,8),Autumn=c(9,10,11),Winter=c(12,1,2),Spring=c(3,4,5))

calendario <- seq(as.Date("1950-01-01"),as.Date("2021-12-31"),by="day")
fechas <- calendario[format(calendario,"%m-%d") != "02-29"]

value <- NA ; value.cat <- NA ; sign <- NA ; sign.cat <- NA ; mix <- list()

for(ind in 1:length(ETCCDI_index)){
  #Load data
  files <- list.files(pattern = paste(ETCCDI_index[ind],"_aemet-5km",sep=""))
  data <- list() ; data.cat <- list()
  
  for(i in 1:length(files)){
    di <- dataInventory(dataset = files[i])
    nc <- loadGridData(dataset=files[i],var=names(di),years=c(1951:2020))
    
    #Aggregation of annual/seasonal indices (climatology: time-series)
    if(length(nc$Dates$start)<=72){
      index2 <- aggregateGrid(nc,aggr.spatial = list(FUN="mean",na.rm=T))
      index.cat <- aggregateGrid(subsetGrid(nc,lonLim = c(0,4),latLim = c(40,43)),
                                 aggr.spatial = list(FUN="mean",na.rm=T))
      
        #Trend in ETCCDI index (if any)
        value  <- lm(index2$Data~c(1:length(unique(getYearsAsINDEX(index2)))))[["coefficients"]][2]
        value.cat  <- lm(index.cat$Data~c(1:length(unique(getYearsAsINDEX(index.cat)))))[["coefficients"]][2]

        sign <- mk.test(index2$Data,continuity = T)$p.value
        sign.cat <- mk.test(index.cat$Data,continuity=T)$p.value
      
      data[[1]] <- data.frame(Trend=value,
                              Season=rep("Annual",length(value)),
                              Sign=sign,
                              Index=ETCCDI_index[ind])
      data.cat[[1]] <- data.frame(Trend=value.cat,
                                  Season=rep("Annual",length(value)),
                                  Sign=sign.cat,
                                  Index=ETCCDI_index[ind])
    }
    if(length(nc$Dates$start)>72){
      for(j in 1:length(seasons)){
        index2 <- redim(aggregateGrid(nc,aggr.s = list(FUN=list("mean",na.rm=T),season=seasons[[j]])),drop=T)
        index.cat <- subsetGrid(index2,lonLim = c(0,4),latLim = c(40,43))
        
        index2 <- aggregateGrid(index2,aggr.spatial = list(FUN="mean",na.rm=T))
        index.cat <- aggregateGrid(index.cat,aggr.spatial = list(FUN="mean",na.rm=T))
        
        value  <- lm(index2$Data~c(1:length(unique(getYearsAsINDEX(index2)))))[["coefficients"]][2]
        value.cat  <- lm(index.cat$Data~c(1:length(unique(getYearsAsINDEX(index.cat)))))[["coefficients"]][2]

        sign <- mk.test(index2$Data,continuity = T)$p.value
        sign.cat <- mk.test(index.cat$Data,continuity=T)$p.value
        
        data[[j]] <- data.frame(Trend=value,
                                Season=rep(names(seasons)[j],length(value)),
                                Sign=sign,
                                Index=ETCCDI_index[ind])
        data.cat[[j]] <- data.frame(Trend=value.cat,
                                    Season=rep(names(seasons)[j],length(value)),
                                    Sign=sign,
                                    Index=ETCCDI_index[ind])
      }}
    
  }
  
  #Organize for ggplot plotting and saving
  datos <- do.call(rbind,data)
  datos$Season <- factor(datos$Season,levels=c("Winter","Autumn","Summer","Spring","Annual"))
  
  datos.cat <- do.call(rbind,data.cat)
  datos.cat$Season <- factor(datos.cat$Season,levels=c("Winter","Autumn","Summer","Spring","Annual"))
  
  # if(length(unique(datos$Season))>1){
  #   datos <- datos[-which(datos$Season=="Annual"),]
  #   datos.cat <- datos.cat[-which(datos.cat$Season=="Annual"),]
  # }
  
  mix[[ind]] <- rbind(datos,datos.cat)
  mix[[ind]]$Region <- rep(c("Spain","Catalunya"),each=dim(datos)[1])
}

data <- do.call("rbind",mix)
data <- data[-which(data$Index%in%c("R95pTOT","R10mm","PRCPTOT")),]
data$Index <- factor(data$Index,levels=rev(c("TN10p","TN90p","TNn","TNx","TR","CSDI","TX10p","TX90p","TXn","TXx","WSDI","Rx1day","Rx5day","CDD","CWD",
                                         "R20mm")))
  png(filename = paste(OUTPUTS,"Trends_aemet-5km_ETCCDI.png",sep=""),res=600,width=3000,height=4000)
  print(ggplot(data=data,aes(x=Season,y=Index,fill=Trend)) + geom_tile() + facet_wrap(~Region,nrow = 2) +
          theme_bw() + ggtitle("AEMET-5km") + 
          scale_color_gradientn("Trend",colours = rev(brewer.pal(n=11,name="RdBu")),limits=c(-0.4,0.4),
                                aesthetics = "fill",na.value = "white") + 
          geom_text(data=subset(data,Sign<=0.05),aes(label="*",fontface="bold")))  
    dev.off()
 
